@@ -187,12 +187,17 @@ class BatchRunner(object):
 
     def run_command(self, command):
         cmd = command['exec'].strip()
-        if self.settings.BATCH_DRY:
-            sys.stderr.write("  Would run '%s'" % cmd)
+        if self.settings.BATCH_VERBOSE:
+            if self.settings.BATCH_DRY:
+                sys.stderr.write("  Would run '%s'" % cmd)
+            else:
+                sys.stderr.write("  Running '%s'" % cmd)
             if command.get('essential', False):
                 sys.stderr.write(" (essential).\n")
             else:
                 sys.stderr.write(" (non-essential).\n")
+
+        if self.settings.BATCH_DRY:
             return
         if command['type'] in ('pre', 'post'):
             try:
@@ -270,8 +275,11 @@ class BatchRunner(object):
         for argset in itertools.product(*argsets):
             rep = argset[-1]
             argset = argset[:-1]
-            sys.stderr.write(" args:%s rep:%02d.\n" % (",".join(argset),rep))
             settings = self.settings.copy()
+            sys.stderr.write(" args:%s rep:%02d" % (",".join(argset),rep))
+            if settings.BATCH_DRY:
+                sys.stderr.write(" (dry run)")
+            sys.stderr.write(".\n")
             settings.FORMAT = 'null'
             settings.BATCH_NAME = batchname
             settings.BATCH_TIME = self.settings.TIME
@@ -299,9 +307,9 @@ class BatchRunner(object):
             else:
                 output_path = os.path.dirname(settings.OUTPUT) or "."
 
-            if settings.BATCH_DRY:
+            if settings.BATCH_DRY and settings.BATCH_VERBOSE:
                 sys.stderr.write("  Would output to: %s.\n" % output_path)
-            elif not os.path.exists(output_path):
+            elif not settings.BATCH_DRY and not os.path.exists(output_path):
                 try:
                     os.makedirs(output_path)
                 except OSError as e:
@@ -316,12 +324,18 @@ class BatchRunner(object):
             self.run_commands(commands, 'pre')
             self.run_commands(commands, 'monitor')
             try:
-                if settings.BATCH_DRY:
-                    sys.stderr.write("  Would run test '%s'.\n" % settings.NAME)
+                if settings.BATCH_VERBOSE:
+                    if settings.BATCH_DRY:
+                        sys.stderr.write("  Would run test '%s'.\n" % settings.NAME)
+                    else:
+                        sys.stderr.write("  Running test '%s'.\n" % settings.NAME)
                     sys.stderr.write("   data_filename=%s\n" % settings.DATA_FILENAME)
                     for k in sorted([i.lower() for i in CONFIG_TYPES.keys()]):
                         if k in b:
                             sys.stderr.write("   %s=%s\n" % (k, b[k]))
+
+                if settings.BATCH_DRY:
+                    self.tests_run += 1
                 else:
                     self.run_test(settings, output_path)
             except KeyboardInterrupt:
@@ -342,9 +356,9 @@ class BatchRunner(object):
                     self.log_fd.close()
                 self.log_fd = None
 
-            if settings.BATCH_DRY:
+            if settings.BATCH_DRY and settings.BATCH_VERBOSE:
                 sys.stderr.write("  Would sleep for %d seconds.\n" % pause)
-            else:
+            elif not settings.BATCH_DRY:
                 time.sleep(pause)
 
 
@@ -421,7 +435,8 @@ class BatchRunner(object):
                 except Exception as e:
                     raise RuntimeError("Error while running batch '%s': %r." % (b, e))
             end_time = datetime.now()
-            sys.stderr.write("Ended batch sequence at %s. Ran %d tests in %s.\n" % (end_time.strftime("%Y-%m-%d %H:%M:%S"),
+            sys.stderr.write("Ended batch sequence at %s. %s %d tests in %s.\n" % (end_time.strftime("%Y-%m-%d %H:%M:%S"),
+                                                                                   "Ran" if not self.settings.BATCH_DRY else 'Would have run',
                                                                                     self.tests_run, (end_time - start_time)))
             return True
         else:
