@@ -23,6 +23,7 @@ import json, sys, csv, math, inspect, os, re
 
 from .util import cum_prob, frange, classname, long_substr
 from .resultset import ResultSet
+from .build_info import DATA_DIR, VERSION
 from functools import reduce
 from itertools import product,cycle,islice
 try:
@@ -256,6 +257,7 @@ class PlotFormatter(Formatter):
         Formatter.__init__(self, settings)
         self.subplot_combine_disabled = False
         self.disable_cleanup = False
+        self.title = None
         try:
             import matplotlib, numpy
             # If saving to file, try our best to set a proper backend for
@@ -274,6 +276,7 @@ class PlotFormatter(Formatter):
                 else:
                     raise RuntimeError("Unrecognised file format for output '%s'" % output)
             import matplotlib.pyplot as plt
+            self.mpl = matplotlib
             self.plt = plt
             self.np = numpy
             self.figure = self.plt.figure()
@@ -310,6 +313,14 @@ class PlotFormatter(Formatter):
                 self.styles.append(dict(marker=m))
 
     def init_plots(self):
+        # Try to detect if a custom matplotlibrc is installed, and if so don't
+        # load our own values.
+        if self.settings.LOAD_MATPLOTLIBRC \
+          and not os.environ['HOME'] in self.mpl.matplotlib_fname() \
+          and not 'MATPLOTLIBRC' in os.environ:
+            self.mpl.rc_file(os.path.join(DATA_DIR, 'matplotlibrc.dist'))
+        self.colours = self.mpl.rcParams['axes.color_cycle']
+
         self.figure.clear()
         if self.settings.FIG_WIDTH is not None:
             self.figure.set_figwidth(self.settings.FIG_WIDTH)
@@ -319,6 +330,9 @@ class PlotFormatter(Formatter):
         self.configs = [self.config]
         getattr(self, '_init_%s_plot' % self.config['type'])()
 
+    def _init_timeseries_combine_plot(self, config=None, axis=None):
+        self._init_timeseries_plot(config, axis)
+
     def _init_timeseries_plot(self, config=None, axis=None):
         if axis is None:
             axis = self.figure.gca()
@@ -332,6 +346,8 @@ class PlotFormatter(Formatter):
             second_axis.yaxis.set_label_position('right')
             second_axis.yaxis.set_offset_position('right')
             second_axis.xaxis.set_visible(False)
+            axis.grid(False)
+            second_axis.grid(False)
             config['axes'] = [axis,second_axis]
         else:
             config['axes'] = [axis]
@@ -350,7 +366,7 @@ class PlotFormatter(Formatter):
                 raise RuntimeError("Plot axis unit mismatch: %s/%s" % (unit[a], s_unit))
             unit[a] = s_unit
 
-        axis.set_xlabel('Time')
+        axis.set_xlabel('Time (s)')
         for i,u in enumerate(unit):
             if 'axis_labels' in config and config['axis_labels'][i]:
                 l = config['axis_labels'][i]
@@ -374,6 +390,9 @@ class PlotFormatter(Formatter):
         self._init_timeseries_plot(config, axis)
         axis.set_xlabel('')
 
+        for a in config['axes']:
+            a.grid(False, axis='x')
+
         self.start_position = 1
 
     def _init_bar_plot(self, config=None, axis=None):
@@ -511,6 +530,9 @@ class PlotFormatter(Formatter):
             return self.dataseries_combine(callback, results, always_colour, config, axis)
 
 
+    def do_timeseries_combine_plot(self, results, config=None, axis=None):
+        return self.do_combine_many_plot(self.do_timeseries_plot, results, config, axis)
+
     def do_timeseries_plot(self, results, config=None, axis=None):
         if len(results) > 1:
             self.combine(self._do_timeseries_plot, results, config, axis)
@@ -576,6 +598,10 @@ class PlotFormatter(Formatter):
             if data[a]:
                 self._do_scaling(config['axes'][a], data[a], btm, top, config['units'][a])
 
+        for a,b in zip(config['axes'], self.settings.BOUNDS_X):
+            a.set_xbound(b)
+        for a,b in zip(config['axes'], self.settings.BOUNDS_Y):
+            a.set_ybound(b)
 
     def do_bar_plot(self, results, config=None, axis=None):
         if config is None:
@@ -593,7 +619,7 @@ class PlotFormatter(Formatter):
         errcol = 'k'
         width = 1.0
 
-        # The median lines are red, so filter out red from the list of colours
+        # The error bars lines are black, so filter out black from the list of colours
         colours = list(islice(cycle([c for c in self.colours if c != errcol]), len(config['series'])))
 
         labels = self._filter_labels([r.label() for r in results])
@@ -631,15 +657,22 @@ class PlotFormatter(Formatter):
             positions = [p-width/2.0 for p in range(pos,pos+group_size)]
             ticks.extend(list(range(pos,pos+group_size)))
             ticklabels.extend(labels)
+            if config.get('colour_by', 'groups') == 'groups':
+                colour = colours[i]
+            else:
+                colour = self.colours[:len(data)]
 
-            bp = config['axes'][a].bar(positions, data, yerr=errors, ecolor=errcol, color=colours[i],
-                                       alpha=0.75, width=width)
+            bp = config['axes'][a].bar(positions, data, yerr=errors, ecolor=errcol, color=colour,
+                                       alpha=0.75, width=width, align='edge')
             if len(config['series']) > 1 or self.settings.PRINT_TITLE:
-                texts.append(config['axes'][0].text(pos+group_size/2, 14, s['label'], ha='center'))
+                texts.append(config['axes'][0].text(pos+group_size/2.0-0.5, 14, s['label'], ha='center'))
 
             config['axes'][a].axvline(x=pos + group_size, color='black', linewidth=0.5, linestyle=':')
             pos += group_size+1
 
+        for a,b in zip(config['axes'], self.settings.BOUNDS_Y):
+            a.set_ybound(b)
+
         min_y,max_y = config['axes'][0].get_ylim()
 
         for t in texts:
@@ -716,6 +749,8 @@ class PlotFormatter(Formatter):
         for i,a in enumerate(config['axes']):
             self._do_scaling(a, all_data[i], 0, 100, config['units'][i], allow_log=False)
 
+        for a,b in zip(config['axes'], self.settings.BOUNDS_Y):
+            a.set_ybound(b)
 
         axis.set_xticks(ticks)
         axis.set_xticks([], minor=True)
@@ -779,6 +814,8 @@ class PlotFormatter(Formatter):
         groups = OrderedDict()
         new_results = []
         filenames = [r.meta('DATA_FILENAME').replace(r.SUFFIX, '') for r in results]
+        for r in self.settings.FILTER_REGEXP:
+            filenames = [re.sub(r, "", f) for f in filenames]
         prefix = long_substr(filenames, prefix_only=True)
         names = map(lambda s: self.serial_regex.sub("", s.replace(prefix, ""), count=1), filenames)
         for i,n in enumerate(names):
@@ -807,6 +844,45 @@ class PlotFormatter(Formatter):
                     x += 1
                 new_results.append(res)
 
+        # groups_points means group by groups, but do per-point combinations, to
+        # e.g. create a data series that is the mean of several others
+        elif group_by == 'groups_points':
+            for k in groups.keys():
+                title = "%s (n=%d)" % (k, len(groups[k])) if self.settings.COMBINE_PRINT_N else k
+                res = ResultSet(TITLE=title, NAME=results[0].meta('NAME'))
+                x_values = []
+                for r in groups[k]:
+                    if len(r.x_values) > len(x_values):
+                        x_values = r.x_values
+                cutoff = config.get('cutoff', None)
+                if cutoff is not None:
+                    res.x_values = [x for x in x_values if x >= cutoff[0] and x <= max(x_values)-cutoff[1]]
+                else:
+                    res.x_values = x_values
+                for s in config['series']:
+                    data = zip_longest(x_values, *[r[s['data']] for r in groups[k]])
+                    new_data = []
+                    for d in data:
+                        if cutoff is None or (d[0] >= cutoff[0] and d[0] <= max(x_values)-cutoff[1]):
+                            new_data.append(self._combine_data(res, s['data'], s.get('combine_mode', 'mean'), None, d=d[1:]))
+                    res.add_result(s['data'], new_data)
+                new_results.append(res)
+
+        # groups_concat means group by groups, but do per-point combinations, to
+        # e.g. create a data series that is the mean of several others
+        elif group_by == 'groups_concat':
+            for k in groups.keys():
+                title = "%s (n=%d)" % (k, len(groups[k])) if self.settings.COMBINE_PRINT_N else k
+                res = ResultSet(TITLE=title, NAME=results[0].meta('NAME'))
+                res.create_series([s['data'] for s in config['series']])
+                x = 0
+                for r in groups[k]:
+                    keys = [s['data'] for s in config['series']]
+                    for p in r.zipped(keys):
+                        res.append_datapoint(x, dict(zip(keys, p[1:])))
+                        x += 1
+                new_results.append(res)
+
         # group_by == 'series' means flip the group and series, so the groups
         # become the entries on the x axis, while the series become the new
         # groups (in the legend)
@@ -861,8 +937,9 @@ class PlotFormatter(Formatter):
 
         return callback(new_results, config, axis)
 
-    def _combine_data(self, resultset, key, combine_mode, cutoff=None):
-        d = resultset[key]
+    def _combine_data(self, resultset, key, combine_mode, cutoff=None, d=None):
+        if d is None:
+            d = resultset[key]
         if cutoff is not None:
             # cut off values from the beginning and end before doing the
             # plot; for e.g. pings that run long than the streams, we don't
@@ -988,6 +1065,9 @@ class PlotFormatter(Formatter):
             # More than an order of magnitude difference; switch to log scale
             axis.set_xscale('log')
 
+        for a,b in zip(config['axes'], self.settings.BOUNDS_X):
+            a.set_xbound(b)
+
     def _filter_dup_vals(self, x_vals, y_vals):
         """Filter out series of identical y-vals, also removing the corresponding x-vals.
 
@@ -1067,13 +1147,18 @@ class PlotFormatter(Formatter):
         if self.settings.ZERO_Y:
             self.xvals.append(0.0)
             self.yvals.append(0.0)
-        axis.set_xlim(min(self.xvals)*0.99, max(self.xvals)*1.1)
-        axis.set_ylim(min(self.yvals)*0.99, max(self.yvals)*1.1)
+        axis.set_xlim(max(min(self.xvals)*0.99,0), max(self.xvals)*1.1)
+        axis.set_ylim(max(min(self.yvals)*0.99,0), max(self.yvals)*1.1)
         if config['invert_x']:
             axis.invert_xaxis()
         if config['invert_y']:
             axis.invert_yaxis()
 
+        for a,b in zip(config['axes'], self.settings.BOUNDS_X):
+            a.set_xbound(b)
+        for a,b in zip(config['axes'], self.settings.BOUNDS_Y):
+            a.set_ybound(b)
+
 
     def _equal_length(self, x, y):
         x_values = self.np.sort([r for r in x if r is not None])
@@ -1148,15 +1233,27 @@ class PlotFormatter(Formatter):
                 self.plt.show()
         else:
             try:
-                self.figure.savefig(self.output, bbox_extra_artists=artists, bbox_inches='tight',
+                if self.plt.get_backend() == 'pdf':
+                    self.save_pdf(self.output, results[0].meta('DATA_FILENAME'), artists)
+                else:
+                    self.figure.savefig(self.output, bbox_extra_artists=artists, bbox_inches='tight',
                                     dpi=self.settings.FIG_DPI)
             except IOError as e:
                 raise RuntimeError("Unable to save output plot: %s" % e)
 
+    def save_pdf(self, filename, data_filename, artists):
+        with self.mpl.backends.backend_pdf.PdfPages(filename) as pdf:
+            pdf.infodict()['Producer'] = 'netperf-wrapper v%s' % VERSION
+            pdf.infodict()['Subject'] = data_filename
+            if self.title:
+                pdf.infodict()['Title'] = self.title.replace("\n", "; ")
+            self.figure.savefig(pdf, bbox_extra_artists=artists, bbox_inches='tight',
+                                dpi=self.settings.FIG_DPI)
+
     def size_legends(self, event=None):
         # For the interactive viewer there's no bbox_extra_artists, so we
         # need to reduce the axis sizes to make room for the legend.
-        if self.settings.PRINT_LEGEND and self.legends:
+        if self.settings.PRINT_LEGEND and not self.settings.HORIZONTAL_LEGEND and self.legends:
             self.figure.canvas.draw() # Legend width is not set before it's drawn
             legend_width = max([l.get_window_extent().width for l in self.legends])
             canvas_width = self.figure.canvas.get_width_height()[0]
@@ -1176,6 +1273,7 @@ class PlotFormatter(Formatter):
         title_y=0.98
         if self.settings.OVERRIDE_TITLE:
             titles.append(self.figure.suptitle(self.settings.OVERRIDE_TITLE, fontsize=14, y=title_y))
+            self.title = self.settings.OVERRIDE_TITLE
         elif self.settings.PRINT_TITLE:
             plot_title = self.settings.DESCRIPTION
             if 'description' in self.config:
@@ -1185,6 +1283,7 @@ class PlotFormatter(Formatter):
             if 'description' in self.config and self.settings.TITLE and not skip_title:
                 y=1.00
             titles.append(self.figure.suptitle(plot_title, fontsize=14, y=title_y))
+            self.title = plot_title
 
         if self.settings.ANNOTATE:
             annotation_string = "Local/remote: %s/%s - Time: %s - Length/step: %ds/%.2fs" % (
@@ -1198,6 +1297,8 @@ class PlotFormatter(Formatter):
         return titles
 
     def _filter_labels(self, labels):
+        for r in self.settings.FILTER_REGEXP:
+            labels = [re.sub(r, "", l) for l in labels]
         if self.settings.FILTER_LEGEND and labels:
             substr = long_substr(labels)
             if len(substr) > 3 and substr != " - ":
@@ -1234,10 +1335,19 @@ class PlotFormatter(Formatter):
             offset_x = 1.02
 
         legends = []
+        if self.settings.HORIZONTAL_LEGEND:
+            bbox = (0.5, -0.12)
+            ncol = len(labels)
+            loc = 'center'
+        else:
+            bbox = (offset_x, 1.0)
+            ncol = 1
+            loc = 'upper left'
         l = axes[0].legend(handles, labels,
-                                bbox_to_anchor=(offset_x, 1.0),
-                                loc='upper left', borderaxespad=0.,
+                                bbox_to_anchor=bbox,
+                                loc=loc, borderaxespad=0.,
                                 prop={'size':'small'},
+                                ncol=ncol,
                                 **kwargs)
 
         # Work around a bug in older versions of matplotlib where the
