@@ -190,8 +190,8 @@ class DitgRunner(ProcessRunner):
 
     def __init__(self, name, settings, duration, interval, **kwargs):
         ProcessRunner.__init__(self, name, settings, **kwargs)
-        host = self.settings.DITG_CONTROL_HOST or self.settings.HOST
-        self.proxy = xmlrpc.ServerProxy("http://%s:%s" % (host, self.settings.DITG_CONTROL_PORT),
+        self.proxy = xmlrpc.ServerProxy("http://%s:%s" % (self.settings.CONTROL_HOST or self.settings.HOST,
+                                                          self.settings.DITG_CONTROL_PORT),
                                         allow_none=True)
         self.ditg_secret = self.settings.DITG_CONTROL_SECRET
         self.duration = duration
@@ -204,7 +204,7 @@ class DitgRunner(ProcessRunner):
             hm = hmac.new(self.ditg_secret.encode(), digestmod=hashlib.sha256)
             hm.update(str(self.duration).encode())
             hm.update(str(interval).encode())
-            params = self.proxy.request_new_test(self.duration, interval, hm.hexdigest())
+            params = self.proxy.request_new_test(self.duration, interval, hm.hexdigest(), self.settings.SAVE_RAW)
             if params['status'] != 'OK':
                 if 'message' in params:
                     raise RuntimeError("Unable to request D-ITG test. Control server reported error: %s" % params['message'])
@@ -254,6 +254,9 @@ class DitgRunner(ProcessRunner):
             self.err += "D-ITG output too much data (%d bytes).\n" % len(data)
             return results
 
+        if 'raw' in res:
+            self.parse_raw(res['raw'])
+
         for line in data.splitlines():
             if not line.strip():
                 continue
@@ -266,6 +269,34 @@ class DitgRunner(ProcessRunner):
 
         return results
 
+    def parse_raw(self, data):
+        raw_values = []
+
+        now = time.time()
+        tzoffset = (datetime.fromtimestamp(now) - datetime.utcfromtimestamp(now)).seconds
+
+        for line in data.splitlines():
+            parts = re.split(r">?\s*", line)
+            vals = dict(zip(parts[::2], parts[1::2]))
+            times = {}
+            for v in ('txTime', 'rxTime'):
+                t,microsec = vals[v].split(".")
+                h,m,s = t.split(":")
+                dt = datetime.utcnow().replace(hour=int(h),
+                                               minute=int(m),
+                                               second=int(s),
+                                               microsecond=int(microsec))
+                times[v] = float(time.mktime(dt.timetuple())) + dt.microsecond / 10**6
+
+            raw_values.append({
+                't': times['rxTime'] + tzoffset,
+                'val': 1000.0 * (times['rxTime'] - times['txTime']),
+                'seq': int(vals['Seq']),
+                'size': int(vals['Size'])
+            })
+
+        self.metadata['RAW_VALUES'] = raw_values
+
 class NetperfDemoRunner(ProcessRunner):
     """Runner for netperf demo mode."""
 
@@ -274,6 +305,7 @@ class NetperfDemoRunner(ProcessRunner):
         pairs."""
 
         result = []
+        raw_values = []
         lines = output.strip().splitlines()
         avg_dur = None
         alpha = 0.5
@@ -289,12 +321,18 @@ class NetperfDemoRunner(ProcessRunner):
                 # the run, or a very short interval giving a very high bandwidth
                 # measurement
                 dur = float(parts[5])
+                time = float(parts[9])
+                value = float(parts[2])
                 if avg_dur is None:
                     avg_dur = dur
 
+                raw_values.append({'dur': dur, 't': time, 'val': value})
+
                 if dur < avg_dur * 10.0 and dur > avg_dur / 10.0:
-                    result.append([float(parts[9]), float(parts[2])])
+                    result.append([time, value])
                     avg_dur = alpha * avg_dur + (1.0-alpha) * dur
+        if self.settings.SAVE_RAW:
+            self.metadata['RAW_VALUES'] = raw_values
         try:
             self.metadata['MEAN_VALUE'] = float(lines[-1])
         except ValueError:
@@ -313,24 +351,30 @@ class RegexpRunner(ProcessRunner):
 
     def parse(self, output):
         result = []
+        raw_values = []
         lines = output.split("\n")
         for line in lines:
             for regexp in self.regexes:
                 match = regexp.match(line)
                 if match:
-                    result.append([float(match.group('time')), float(match.group('value'))])
+                    result.append([float(match.group('t')), float(match.group('val'))])
+                    raw_values.append(match.groupdict())
                     break # only match one regexp per line
+        if self.settings.SAVE_RAW:
+            self.metadata['RAW_VALUES'] = raw_values
         return result
 
 class PingRunner(RegexpRunner):
     """Runner for ping/ping6 in timestamped (-D) mode."""
 
-    regexes = [re.compile(r'^\[(?P<time>[0-9]+\.[0-9]+)\].*time=(?P<value>[0-9]+(?:\.[0-9]+)?) ms$'),
-               re.compile(r'^\[(?P<time>[0-9]+\.[0-9]+)\].*:.*, (?P<value>[0-9]+(?:\.[0-9]+)?) ms \(.*\)$')]
+    # For some reason some versions of ping output icmp_req and others icmp_seq
+    # for sequence numbers.
+    regexes = [re.compile(r'^\[(?P<t>[0-9]+\.[0-9]+)\](?:.*icmp_.eq=(?P<seq>[0-9]+))?.*time=(?P<val>[0-9]+(?:\.[0-9]+)?) ms$'),
+               re.compile(r'^\[(?P<t>[0-9]+\.[0-9]+)\].*:(?: \[(?P<seq>[0-9]+)\])?.*, (?P<val>[0-9]+(?:\.[0-9]+)?) ms \(.*\)$')]
 
 class HttpGetterRunner(RegexpRunner):
 
-    regexes = [re.compile(r'^\[(?P<time>[0-9]+\.[0-9]+)\].*in (?P<value>[0-9]+(?:\.[0-9]+)?) seconds.$')]
+    regexes = [re.compile(r'^\[(?P<t>[0-9]+\.[0-9]+)\].*in (?P<val>[0-9]+(?:\.[0-9]+)?) seconds.$')]
 
 class IperfCsvRunner(ProcessRunner):
     """Runner for iperf csv output (-y C), possibly with unix timestamp patch."""
@@ -481,7 +525,7 @@ class ComputingRunner(object):
             else:
                 new_res.append(self.compute(values))
 
-        meta = res.meta('SERIES_META')
+        meta = res.meta('SERIES_META') if 'SERIES_META' in res.meta() else {}
         meta[self.name] = {}
         for mk in self.supported_meta:
             vals = []
